
# q-expansion of newform 170.2.c.b, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 170
weight = 2
poly_data = [8, -20, 25, -4, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0], 1], [[100, -121, 10, -25, -2, -5], 121], [[223, 0, -14, 35, 27, 7], 121], [[258, -605, 50, -4, -10, -25], 242], [[574, -1331, 372, 38, -26, -65], 242]]

hecke_ring_character_values = [[137, [-1, 0, 0, 0, 0, 0]], [71, [1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 1, 0], [0, -1, 1, 0, 0, 1], [0, 0, -1, -1, 0, 0], [0, -1, 1, 0, 0, 0], [2, 0, 0, 0, 0, 0], [0, -1, 1, 0, 0, 1], [0, 0, 0, 0, 1, 0], [0, -1, -1, 1, 0, 0], [0, 1, -1, 0, 4, 0], [-6, -2, -2, -1, 0, 0], [2, 2, 2, 3, 0, 0], [0, 1, -1, 0, 4, 4], [6, 0, 0, 2, 0, 0], [0, 2, -2, 0, -2, -2], [0, -1, 1, 0, -6, -3], [0, 3, -3, 0, 0, -5], [-2, 1, 1, 3, 0, 0], [-2, 0, 0, 3, 0, 0], [0, -2, 2, 0, -4, 2], [-2, -2, -2, -5, 0, 0], [0, -1, 1, 0, 0, 1], [0, 1, 1, 4, 0, 0], [0, -2, 2, 0, 6, -2], [2, -1, -1, -3, 0, 0], [0, 1, -1, 0, 0, -3], [2, 2, 2, -2, 0, 0], [0, 2, -2, 0, 4, -4], [0, -2, 2, 0, -6, 6], [-2, 2, 2, -3, 0, 0], [0, 1, -1, 0, -8, -1], [0, 3, -3, 0, -10, -1], [10, 0, 0, 4, 0, 0], [0, -4, 4, 0, 4, 4], [6, 6, 6, 2, 0, 0], [-10, 0, 0, 0, 0, 0], [0, -6, -6, 0, 0, 0], [0, -6, 6, 0, -2, 4], [0, 0, 0, 0, -10, 2], [0, 5, -5, 0, 4, -2], [0, -1, 1, 0, 16, 4], [6, 6, 6, 2, 0, 0], [-12, -1, -1, -6, 0, 0], [8, 4, 4, 4, 0, 0], [0, 4, -4, 0, -2, 0], [0, 5, -5, 0, 8, -10], [6, 0, 0, 3, 0, 0], [-6, 2, 2, -6, 0, 0], [0, -3, 3, 0, 6, -3], [0, -1, 1, 0, 8, 9], [-6, 2, 2, 0, 0, 0], [0, -3, 3, 0, 0, -1], [-4, 6, 6, 2, 0, 0], [-6, -8, -8, -6, 0, 0], [6, -2, -2, -6, 0, 0], [0, 4, -4, 0, 12, -4], [0, -1, 1, 0, -2, -5], [-2, -2, -2, 1, 0, 0], [0, -2, -2, 6, 0, 0], [0, -5, 5, 0, 12, 2], [10, -7, -7, -9, 0, 0], [0, 3, -3, 0, 4, 7], [0, 5, -5, 0, 8, -9], [0, -2, 2, 0, -22, -2], [4, 9, 9, 12, 0, 0], [0, 0, 0, 0, -22, -4], [0, 3, -3, 0, -4, -8], [-2, 9, 9, -1, 0, 0], [0, -3, 3, 0, 4, -5], [0, 1, -1, 0, 4, -1], [-2, 2, 2, 12, 0, 0], [0, 6, -6, 0, 2, -6], [-4, 2, 2, -4, 0, 0], [0, 1, -1, 0, -8, -2], [0, -2, 2, 0, -18, -4], [-22, -4, -4, -2, 0, 0], [0, -7, 7, 0, -14, 13], [14, -2, -2, 4, 0, 0], [0, 7, -7, 0, 0, -2], [6, -2, -2, 4, 0, 0], [10, 1, 1, -1, 0, 0], [-10, 4, 4, 6, 0, 0], [10, 2, 2, 2, 0, 0], [0, 3, 3, -4, 0, 0], [0, -6, 6, 0, 2, 2], [8, -7, -7, -4, 0, 0], [0, -6, 6, 0, -12, 10], [-10, 2, 2, 8, 0, 0], [0, -2, 2, 0, -26, 6], [-6, -4, -4, -10, 0, 0], [0, -7, 7, 0, 26, 3], [0, 2, -2, 0, -16, -10], [-10, -6, -6, -9, 0, 0], [0, 3, -3, 0, 0, -6], [20, 3, 3, 1, 0, 0], [-10, -10, -10, -10, 0, 0], [0, 9, -9, 0, -24, -6], [14, -2, -2, -6, 0, 0], [10, -4, -4, -2, 0, 0], [0, -2, 2, 0, -4, -2], [-4, 9, 9, -2, 0, 0], [0, 11, -11, 0, 0, -13], [0, -3, 3, 0, 8, 7], [0, 2, -2, 0, 2, 10], [-8, -3, -3, -1, 0, 0], [-22, 2, 2, -4, 0, 0], [0, -4, 4, 0, 0, -8], [0, -12, 12, 0, 12, 0], [0, -10, 10, 0, 0, 2], [4, 0, 0, 12, 0, 0], [6, -4, -4, -4, 0, 0], [0, 7, -7, 0, -8, 4], [0, -1, 1, 0, 4, -7], [0, -5, 5, 0, 16, 9], [6, -2, -2, 0, 0, 0], [12, 4, 4, 6, 0, 0], [18, 2, 2, -4, 0, 0], [0, -4, 4, 0, -18, -6], [0, -3, 3, 0, 2, 19], [0, 17, -17, 0, -20, -6], [-10, -5, -5, -15, 0, 0], [-22, 8, 8, 10, 0, 0], [0, 1, -1, 0, 8, 17], [0, 13, -13, 0, -12, -12], [0, -1, 1, 0, 4, -7], [-10, 6, 6, 18, 0, 0], [22, -2, -2, 2, 0, 0], [10, 2, 2, 3, 0, 0], [10, 14, 14, 1, 0, 0], [0, 7, -7, 0, -10, -7], [0, 8, -8, 0, 14, 12], [4, -11, -11, 3, 0, 0], [0, -11, 11, 0, 24, 8], [-14, -2, -2, 9, 0, 0], [0, 15, -15, 0, -12, -5], [-6, 12, 12, 4, 0, 0], [8, -9, -9, -7, 0, 0], [0, -4, 4, 0, 34, 0], [0, -9, 9, 0, 12, 3], [0, -4, 4, 0, 2, 8], [6, -14, -14, -8, 0, 0], [14, -4, -4, 10, 0, 0], [-18, -12, -12, -13, 0, 0], [0, -5, 5, 0, 24, 14], [0, 14, -14, 0, -10, -10], [-2, -8, -8, -8, 0, 0], [10, 4, 4, -9, 0, 0], [0, -5, 5, 0, 4, -6], [0, -11, 11, 0, 0, 5], [22, 9, 9, 7, 0, 0], [0, -10, 10, 0, 8, 16], [0, 1, -1, 0, -12, 6], [-14, -8, -8, 0, 0, 0], [0, 14, -14, 0, -4, -6], [0, -3, 3, 0, 16, 6], [0, -7, 7, 0, -24, -3], [-20, -1, -1, 0, 0, 0], [-20, -2, -2, 2, 0, 0], [2, -6, -6, -8, 0, 0], [0, 2, -2, 0, -38, -6], [26, 0, 0, -3, 0, 0], [0, 1, -1, 0, 8, 1], [0, -10, 10, 0, -2, 6], [0, 0, 0, 0, -16, 8], [-6, -9, -9, 5, 0, 0], [0, 18, -18, 0, -4, -18], [0, 3, -3, 0, 16, -2], [6, -16, -16, -17, 0, 0], [0, -7, 7, 0, -12, 8]]
